/* RandomOdds.c **************************************************************/

// Copyright (C) 2005, Brian 'Psychochild' Green
// All rights reserved.                          
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   1. Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//   2. Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//   3. The names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "RandomOdds.h"
// Included for time.
#include <time.h>

RandomOdds::RandomOdds(int iSuccesses /*= 1*/, int iOutOf /*= 2*/)
{
   if (!ResetOdds(iSuccesses, iOutOf))
   {
      ResetOdds(1,2);
   }

   // Seed the random number generator.
   mpRandom = new MTRand((unsigned long)time(NULL));

   return;
}

RandomOdds::~RandomOdds()
{
   delete mpRandom;

   return;
}

bool RandomOdds::ResetOdds(int iSuccesses /*= -1*/, int iOutOf /*= -1*/)
{
   // local variables for storing information.
   int liSuccesses;
   int liOutOf;

   if (iSuccesses > 0)
   {
      liSuccesses = iSuccesses;
   }
   else
   {
      liSuccesses = miSuccesses;
   }

   if (iOutOf > 0)
   {
      liOutOf = iOutOf;
   }
   else
   {
      liOutOf = miOutOf;
   }

   if (liSuccesses > liOutOf)
   {
      // Bad values, don't change anything.
      return false;
   }

   miSuccesses = liSuccesses;
   miOutOf = liOutOf;
   miNumSuccesses = 0;
   miNumRolls = 0;

   return true;
}

bool RandomOdds::GetChance()
{
   // Local variable containing current odds.
   double ldOdds;
   double ldRandom;

   // Do we need to reset the odds because we've already rolled enough?
   if (miNumRolls >= miOutOf)
   {
      ResetOdds();
   }

   // Calculate current odds.
   ldOdds = (double)(miSuccesses - miNumSuccesses) / (double)(miOutOf - miNumRolls);
   ldRandom = mpRandom->rand();

   miNumRolls++;

   if (ldRandom > ldOdds)
   {
      return false;
   }

   miNumSuccesses++;

   return true;
}