/* RandomOdds.h ***************************************************************

/* This is a definition of a class that takes certain odds and provides
** an object that will give you a series of boolean values based on those odds.
** For example, if you want the odds to be 5 out of 13, then in a group of
** 13 consecutive calls you will get exactly 5 successes.  Note that this does
** not mean that you will have 5 successes in *any* 13 calls.  This may be an
** interesting extension.
*/

// Copyright (C) 2005, Brian 'Psychochild' Green
// All rights reserved.                          
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   1. Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//   2. Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//   3. The names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef RANDOMODDS_H
#define RANDOMODDS_H

/* Includes */

// Included for random numbers
#include "MersenneTwister.h"

/* classes */

class RandomOdds
{
public:
   RandomOdds(int iSuccesses = 1, int iOutOf = 2);
   ~RandomOdds();

   // Resets odds.  If parameter values are non-positive, it will not change
   //  that value.  Call with default parameters to reset the counts.
   // Conditions: iSuccesses <= iOutOf
   // Returns: true if reset, false if not.
   bool ResetOdds(int iSuccesses = -1, int iOutOf = -1);

   // Call to get either a true or false value, depending on the number of
   //  successes and failures before.
   bool GetChance();

private:
   // The set odds
   int miSuccesses;
   int miOutOf;
   // The current tally
   int miNumSuccesses;
   int miNumRolls;
   // Our random number generator
   MTRand* mpRandom;
};

#endif // #ifndef RANDOMODDS_H